/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

#ifndef __PAF_ANIMATION_H__
#define __PAF_ANIMATION_H__
#include "pafcommon.h"
#include "pafclock.h"
#include "paftransformation.h"
G_BEGIN_DECLS

typedef struct _PafAnimationPrivate PafAnimationPrivate;

typedef struct {
    GObject parent;

    PafAnimationPrivate *priv;
} PafAnimation;

typedef struct {
    GObjectClass parent;
} PafAnimationClass;

#define PAF_TYPE_ANIMATION (paf_animation_get_type ())
#define PAF_ANIMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
                                                        PAF_TYPE_ANIMATION, \
                                                        PafAnimation))
#define PAF_ANIMATION_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_CAST ((klass), PAF_TYPE_ANIMATION, PafAnimationClass))
#define PAF_IS_ANIMATION(obj) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_ANIMATION))
#define PAF_IS_ANIMATION_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_TYPE ((klass), PAF_TYPE_ANIMATION))
#define PAF_ANIMATION_GET_CLASS(obj) \
    (G_TYPE_INSTANCE_GET_CLASS ((obj), PAF_TYPE_ANIMATION, PafAnimationClass))

GType           paf_animation_get_type (void);

PafAnimation *  paf_animation_new (PafClock *clock);

PafAnimation *  paf_animation_new_from_duration (guint duration);

void            paf_animation_add_transformation (PafAnimation *animation,
                                             PafTransformation *transformation);

void            paf_animation_add_transformations (PafAnimation *animation,
                                      PafTransformation *first_transformation,
                                      ...) G_GNUC_NULL_TERMINATED;

void            paf_animation_add_transformations_valist (
                                     PafAnimation *animation,
                                     va_list var_args);

void            paf_animation_add_transformation_list (PafAnimation *animation,
                                                       GSList *list);

gboolean        paf_animation_remove_transformation (PafAnimation *animation,
                                             PafTransformation *transformation);

gboolean        paf_animation_has_transformation (PafAnimation *animation,
                                    const PafTransformation *transformation);

PafClock       *paf_animation_get_clock (PafAnimation *animation);

PafState        paf_animation_get_state (PafAnimation *animation);

void            paf_animation_set_state (PafAnimation *animation,
                                         PafState state);

G_END_DECLS

#endif /* __PAF_ANIMATION_H__ */
