/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Clock
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */
#ifndef __PAF_CLOCK_H__
#define __PAF_CLOCK_H__

#include "pafcommon.h"
#include "paftransformation.h"
#include "pafticker.h"

G_BEGIN_DECLS

typedef struct _PafClockPrivate PafClockPrivate;

struct _PafClock{
    GObject parent;

    PafClockPrivate *priv;
};

typedef struct {
    GObjectClass parent;
} PafClockClass;

#define PAF_TYPE_CLOCK (paf_clock_get_type ())
#define PAF_CLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
                                                    PAF_TYPE_CLOCK, PafClock))
#define PAF_CLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                         PAF_TYPE_CLOCK, \
                                                         PafClockClass))
#define PAF_IS_CLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_CLOCK))
#define PAF_IS_CLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                            PAF_TYPE_CLOCK))
#define PAF_CLOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                             PAF_TYPE_CLOCK, \
                                                             PafClockClass))
GType       paf_clock_get_type (void);

PafClock *  paf_clock_new (guint duration);

PafClock *  paf_clock_new_full (guint duration,
                                gfloat repeat_count,
                                gboolean auto_reverse,
                                PafTransformation *time_transformation,
                                PafTicker *ticker);

gboolean    paf_clock_tick     (PafClock *clock);

guint       paf_clock_get_duration      (PafClock *clock);
void        paf_clock_set_duration      (PafClock *clock, guint duration);
gfloat      paf_clock_get_repeat_count  (PafClock *clock);
void        paf_clock_set_repeat_count  (PafClock *clock, gfloat repeat_count);
gboolean    paf_clock_get_auto_reverse  (PafClock *clock);
void        paf_clock_set_auto_reverse  (PafClock *clock,
                                         gboolean auto_reverse);
PafTransformation * paf_clock_get_time_transformation (PafClock *clock);
void        paf_clock_set_time_transformation (PafClock *clock,
                                       PafTransformation *time_transformation);
PafState    paf_clock_get_state         (PafClock *clock);
void        paf_clock_set_state         (PafClock *clock,
                                         PafState state);
PafTicker  *paf_clock_get_ticker        (PafClock *clock);
void        paf_clock_set_ticker        (PafClock *clock, PafTicker *ticker);

G_END_DECLS
#endif /* __PAF_CLOCK_H__ */
