/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */
#ifndef __PAF_COMMON_H__
#define __PAF_COMMON_H__

#include <glib-object.h>
#include <math.h>
#include "pafmarshal.h"
#include "pafenumtypes.h"

typedef struct _PafClock  PafClock;
typedef struct _PafTicker PafTicker;

/**
 * PafState:
 * @PAF_STATE_STOPPED: the clock is ignoring all ticks, and will reinitialise
 * the time count used to know where we are regarding PafClock:duration when
 * changed to PAF_STATE_RUNNING.
 * @PAF_STATE_RUNNING: the clock is active and doing its job.
 * @PAF_STATE_PAUSED: the clock is ignoring all ticks.
 * @PAF_LAST_STATE: here for implementation purposes, to check for the
 * validity of a PafClockStatus.
 *
 * Type used by the #PafClock:state property. The #PafAnimation:state also makes
 * use of it, but it is actually linked to the #PafClock:state of the underlying
 * #PafClock.
 */
typedef enum {
    PAF_STATE_STOPPED,
    PAF_STATE_RUNNING,
    PAF_STATE_PAUSED,
    PAF_LAST_STATE
} PafState;

/**
 * PAF_INFINITY:
 *
 * A #gfloat constant representing infinity. Particularly useful for
 * #PafClock:repeat-count, but may have other applications
 */
#ifdef INFINITY
#define PAF_INFINITY INFINITY
#else
/* heard this is infinity in IEEE 754. At least it's big. Stolen from glibc's
 * HUGE_VALF */
typedef union { unsigned char __c[4]; float __f; } _paf_infinityf_t;

# if G_BYTE_ORDER == G_BIG_ENDIAN
#  define __INFINITY_bytes	{ 0x7f, 0x80, 0, 0 }
# endif
# if G_BYTE_ORDER == G_LITTLE_ENDIAN
#  define __INFINITY_bytes	{ 0, 0, 0x80, 0x7f }
# endif

static G_GNUC_UNUSED _paf_infinityf_t _paf_infinityf = { __INFINITY_bytes };
# define PAF_INFINITY	(_paf_infinityf.__f)
#endif

gboolean paf_accumulate_booleans (GSignalInvocationHint *invocation_hint,
                                  GValue *_return_accumulator,
                                  const GValue *_handler_return,
                                  gpointer data);

#endif /* __PAF_COMMON_H__ */
