/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Implicit Animator
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

/**
 * SECTION:pafimplicitanimator
 * @short_description: FIXME
 * @see_also: #PafAnimation and #PafInterpolation.
 *
 * <refsect2>
 * FIXME
 * </refsect2>
 */

#include "pafimplicitanimator.h"


G_DEFINE_TYPE (PafImplicitAnimator, paf_implicit_animator, G_TYPE_OBJECT);

#define PAF_IMPLICIT_ANIMATOR_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), PAF_TYPE_IMPLICIT_ANIMATOR, \
                                  PafImplicitAnimatorPrivate))

struct _PafImplicitAnimatorPrivate {
    /* Properties */
    GObject *target;
    GType time_transformation_type;
    GType interpolation_type;
    guint duration;
    gfloat repeat_count;
    gboolean auto_reverse;
    PafTicker *ticker;
};

/* property IDs */
enum {
    TARGET=1,
    TIME_TRANSFORMATION_TYPE,
    INTERPOLATION_TYPE,
    DURATION,
    REPEAT_COUNT,
    AUTO_REVERSE,
    TICKER,
    LAST_PROPERTY
};

/* GObject stuff */
static void
_paf_implicit_animator_set_property (GObject      *object,
                                    guint         property_id,
                                    const GValue *value,
                                    GParamSpec   *pspec)
{
}

static void
_paf_implicit_animator_get_property (GObject      *object,
                                    guint         property_id,
                                    GValue       *value,
                                    GParamSpec   *pspec)
{
}


static void
paf_implicit_animator_dispose (PafImplicitAnimator *self)
{
}

static void
paf_implicit_animator_finalize (PafImplicitAnimator *self)
{
}

static void
paf_implicit_animator_init (PafImplicitAnimator *self)
{
    PafImplicitAnimatorPrivate *priv;

    priv = PAF_IMPLICIT_ANIMATOR_GET_PRIVATE (self);
}

static void
paf_implicit_animator_class_init (PafImplicitAnimatorClass *self_class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (self_class);
    GParamSpec *param_spec = NULL;

    g_type_class_add_private (self_class, sizeof (PafImplicitAnimatorPrivate));

    object_class->set_property = _paf_implicit_animator_set_property;
    object_class->get_property = _paf_implicit_animator_get_property;

    param_spec = g_param_spec_object ("target", "Animated object",
                                      "Object that gets its properties "
                                      "animated through the implicit animator",
                                      G_TYPE_OBJECT,
                                      G_PARAM_READWRITE
                                      | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (object_class, TARGET, param_spec);

    param_spec = g_param_spec_gtype ("time-transformation-type",
                                     "Time modifier type",
                                     "Which transformation to apply to the "
                                     "evolution of time",
                                     PAF_TYPE_TRANSFORMATION,
                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class,
                                     TIME_TRANSFORMATION_TYPE, param_spec);

    param_spec = g_param_spec_gtype ("interpolation-type", "Interpolation type",
                                     "Which interpolation to use for future "
                                     "property animations",
                                     PAF_TYPE_INTERPOLATION,
                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class,
                                     INTERPOLATION_TYPE, param_spec);

    param_spec = g_param_spec_uint ("duration", "Duration",
                                    "Default duration of an animated change of "
                                    "value (considering repeat-count is 1)",
                                    0, G_MAXUINT, 1000,
                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class, DURATION, param_spec);

    param_spec = g_param_spec_float ("repeat-count", "Repeat count",
                                    "Default number of times the animated "
                                    "change should happen",
                                    0.0f, G_MAXFLOAT, 1.0f,
                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class, REPEAT_COUNT, param_spec);

    param_spec = g_param_spec_boolean ("auto-reverse", "Auto reverse",
                                       "Whether to reverse time direction when "
                                       "repeat-count is more than 1",
                                       FALSE,
                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class, AUTO_REVERSE, param_spec);

    param_spec = g_param_spec_object ("ticker", "Ticker",
                                      "The PafTicker that decides when "
                                      "properties are updated",
                                      G_TYPE_OBJECT,
                                      G_PARAM_READWRITE
                                      | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (object_class, TICKER, param_spec);

    object_class->dispose = (void (*) (GObject *object)) paf_implicit_animator_dispose;
    object_class->finalize = (void (*) (GObject *object)) paf_implicit_animator_finalize;
}

/* public methods */

PafImplicitAnimator *
paf_implicit_animator_new (GObject *target)
{
    return NULL;
}

void
paf_implicit_animator_set_property (PafImplicitAnimator *animator,
                                    const gchar *property_name,
                                    GValue *value)
{
}

void
paf_implicit_animator_set_property_values (PafImplicitAnimator *animator,
                                           const gchar *property_name,
                                           GValue *first_value,
                                           ...)
{
}
void
paf_implicit_animator_set_property_values_va_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                va_list var_args)
{
}

void
paf_implicit_animator_set_property_value_list (PafImplicitAnimator *animator,
                                               const gchar *property_name,
                                               GSList *values)
{
}

void
paf_implicit_animator_set_property_key_values (PafImplicitAnimator *animator,
                                               const gchar *property_name,
                                               PafKeyValue *first_key_value,
                                               ...)
{
}
void
paf_implicit_animator_set_property_key_values_va_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                va_list var_args)
{
}

void
paf_implicit_animator_set_property_key_value_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                GSList *key_values)
{
}

