/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Implicit Animator
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

#ifndef __PAF_IMPLICIT_ANIMATOR__
#define __PAF_IMPLICIT_ANIMATOR__

#include "pafcommon.h"
#include "pafanimation.h"
#include "pafinterpolation.h"

G_BEGIN_DECLS

typedef struct _PafImplicitAnimatorPrivate PafImplicitAnimatorPrivate;

typedef struct {
    GObject parent;

    PafImplicitAnimatorPrivate *priv;
} PafImplicitAnimator;

typedef struct {
    GObjectClass parent;
} PafImplicitAnimatorClass;

#define PAF_TYPE_IMPLICIT_ANIMATOR (paf_implicit_animator_get_type ())
#define PAF_IMPLICIT_ANIMATOR(obj) \
    (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAF_TYPE_IMPLICIT_ANIMATOR, \
                                 PafImplicitAnimator))
#define PAF_IMPLICIT_ANIMATOR_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_CAST ((klass), PAF_TYPE_IMPLICIT_ANIMATOR, \
                              PafImplicitAnimatorClass))
#define PAF_IS_IMPLICIT_ANIMATOR(obj) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_IMPLICIT_ANIMATOR))
#define PAF_IS_IMPLICIT_ANIMATOR_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_TYPE ((klass), PAF_TYPE_IMPLICIT_ANIMATOR))
#define PAF_IMPLICIT_ANIMATOR_GET_CLASS(obj) \
    (G_TYPE_INSTANCE_GET_CLASS ((obj), PAF_TYPE_IMPLICIT_ANIMATOR, \
                                PafImplicitAnimatorClass))

GType paf_implicit_animator_get_type (void);

PafImplicitAnimator * paf_implicit_animator_new (GObject *target);


void paf_implicit_animator_set_property (PafImplicitAnimator *animator,
                                         const gchar *property_name,
                                         GValue *value);

void paf_implicit_animator_set_property_values (PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                GValue *first_value,
                                                ...) G_GNUC_NULL_TERMINATED;
void paf_implicit_animator_set_property_values_va_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                va_list var_args);

void paf_implicit_animator_set_property_value_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                GSList *values);

void paf_implicit_animator_set_property_key_values (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                PafKeyValue *first_key_value,
                                                ...) G_GNUC_NULL_TERMINATED;
void paf_implicit_animator_set_property_key_values_va_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                va_list var_args);

void paf_implicit_animator_set_property_key_value_list (
                                                PafImplicitAnimator *animator,
                                                const gchar *property_name,
                                                GSList *key_values);

G_END_DECLS

#endif /* __PAF_IMPLICIT_ANIMATOR__ */
