/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Interpolation
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

/**
 * SECTION:pafinterpolation
 * @short_description: FIXME
 * @see_also: #PafTransformation, #PafModifier and #PafOperators.
 *
 * <refsect2>
 * FIXME
 * </refsect2>
 */

#include "pafinterpolation.h"

G_DEFINE_TYPE (PafInterpolation, paf_interpolation, PAF_TYPE_TRANSFORMATION);

#define PAF_INTERPOLATION_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), PAF_TYPE_INTERPOLATION, \
                                  PafInterpolationPrivate))

struct _PafInterpolationPrivate {
    /* Properties */
    GType interpolated_type;
    PafOperators *operators;
    gboolean jump_on_modification;
};

/* PafInterpolation properties */
enum {
    INTERPOLATED_TYPE=1,
    OPERATORS,
    JUMP_ON_CHANGE,
    LAST_PROPERTY
};

/* private methods */

gboolean _paf_interpolation_add_modifier (PafInterpolation *interpolation,
                                          PafModifier *modifier)
{
    return FALSE;
}

gboolean _paf_interpolation_remove_modifier (PafInterpolation *interpolation,
                                          PafModifier *modifier)
{
    return FALSE;
}

gboolean _paf_interpolation_have_modifier (PafInterpolation *interpolation,
                                          PafModifier *modifier)
{
    return FALSE;
}

gboolean _paf_interpolation_update (PafInterpolation *interpolation,
                                    gfloat fraction)
{
    return FALSE;
}

/* GObject stuff */

static void
paf_interpolation_set_property (GObject *object,
                                guint property_id,
                                const GValue *value,
                                GParamSpec *param_spec)
{
}

static void
paf_interpolation_get_property (GObject *object,
                                guint property_id,
                                GValue *value,
                                GParamSpec *param_spec)
{
}

static void
paf_interpolation_dispose (PafInterpolation *self)
{
}

static void
paf_interpolation_finalize (PafInterpolation *self)
{
}

static void
paf_interpolation_init (PafInterpolation *self)
{
    PafInterpolationPrivate *priv;

    priv = PAF_INTERPOLATION_GET_PRIVATE (self);
}

static void
paf_interpolation_class_init (PafInterpolationClass *self_class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (self_class);
    GParamSpec *param_spec = NULL;

    g_type_class_add_private (self_class, sizeof (PafInterpolationPrivate));
    object_class->set_property = paf_interpolation_set_property;
    object_class->get_property = paf_interpolation_get_property;

    param_spec = g_param_spec_gtype ("interpolated-type", "Interpolated Type",
                                     "The type to be interpolated.",
                                     G_TYPE_NONE,
                                     G_PARAM_READWRITE
                                     | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (object_class, INTERPOLATED_TYPE,
                                     param_spec);

    param_spec = g_param_spec_object ("operators", "Operators",
                                      "Operators defined for the interpolated "
                                      "type.",
                                      PAF_TYPE_OPERATORS,
                                      G_PARAM_READWRITE
                                      | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (object_class, OPERATORS, param_spec);

    param_spec = g_param_spec_boolean ("jump-on-modification",
                                       "Jump On modification",
                                       "Whether a discontinuity (\"jump\") "
                                       "should happen when modifying the "
                                       "key values.",
                                       FALSE,
                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
    g_object_class_install_property (object_class, JUMP_ON_CHANGE, param_spec);

    object_class->dispose =
        (void (*) (GObject *object)) paf_interpolation_dispose;
    object_class->finalize =
        (void (*) (GObject *object)) paf_interpolation_finalize;
}

/* Public methods */


void
paf_interpolation_add_key_value (PafInterpolation *interpolation,
                                 PafKeyValue *key_value)
{
}

void
paf_interpolation_add_key_values (PafInterpolation *interpolation,
                                  PafKeyValue *first_key_value,
                                  ...)
{
}

void
paf_interpolation_add_key_values_valist (PafInterpolation *interpolation,
                                         va_list var_args)
{
}

void
paf_interpolation_add_key_value_list (PafInterpolation *interpolation,
                                      GSList *key_values)
{
}

void
paf_interpolation_add_key_value_from_value (
                                            PafInterpolation *interpolation,
                                            gfloat fraction,
                                            GValue *value)
{
}

void
paf_interpolation_add_key_value_from_float (
                                            PafInterpolation *interpolation,
                                            gfloat fraction,
                                            gfloat value)
{
}

void
paf_interpolation_add_key_value_from_int (
                                          PafInterpolation *interpolation,
                                          gfloat fraction,
                                          gint value)
{
}

void
paf_interpolation_modify_key_value (PafInterpolation *interpolation,
                                    gfloat fraction, PafKeyValue *key_value)
{
}

void
paf_interpolation_remove_key_value_from_fraction (
                                             PafInterpolation *interpolation,
                                             gfloat fraction)
{
}

void
paf_interpolation_remove_key_value (PafInterpolation *interpolation,
                                    PafKeyValue *value)
{
}

void
paf_interpolation_get_surrounding_key_values (
                                              PafInterpolation *interpolation,
                                              gfloat fraction,
                                              PafKeyValue **before,
                                              PafKeyValue **after)
{
}

void
paf_interpolation_set_key_value_options (PafInterpolation *interpolation,
                                         PafKeyValue *key_value,
                                         GValue *options)
{
}

void
paf_interpolation_set_key_value_options_from_fraction (
                                              PafInterpolation *interpolation,
                                              gfloat fraction,
                                              GValue *options)
{
}

