/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Interpolation
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

#ifndef __PAF_INTERPOLATION_H__
#define __PAF_INTERPOLATION_H__

#include "pafcommon.h"
#include "pafoperators.h"
#include "paftransformation.h"
#include "pafoperators.h"
#include "pafkeyvalue.h"

G_BEGIN_DECLS

typedef struct _PafInterpolationPrivate PafInterpolationPrivate;

typedef struct {
    GObject parent;

    PafInterpolationPrivate *priv;
} PafInterpolation;

typedef struct {
    PafTransformationClass parent;
} PafInterpolationClass;

#define PAF_TYPE_INTERPOLATION (paf_interpolation_get_type ())
#define PAF_INTERPOLATION(obj) \
    (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAF_TYPE_INTERPOLATION, \
                                 PafInterpolation))
#define PAF_INTERPOLATION_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_CAST ((klass), PAF_TYPE_INTERPOLATION, \
                              PafInterpolationClass))
#define PAF_IS_INTERPOLATION(obj) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_INTERPOLATION))
#define PAF_IS_INTERPOLATION_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_TYPE ((klass), PAF_TYPE_INTERPOLATION))
#define PAF_INTERPOLATION_GET_CLASS(obj) \
    (G_TYPE_INSTANCE_GET_CLASS ((obj), PAF_TYPE_INTERPOLATION, \
                                PafInterpolationClass))

GType paf_interpolation_get_type (void);

void paf_interpolation_add_key_value (PafInterpolation *interpolation,
                                      PafKeyValue *key_value);

void paf_interpolation_add_key_values (PafInterpolation *interpolation,
                                       PafKeyValue *first_key_value,
                                       ...) G_GNUC_NULL_TERMINATED;

void paf_interpolation_add_key_values_valist (PafInterpolation *interpolation,
                                              va_list var_args);

void paf_interpolation_add_key_value_list (PafInterpolation *interpolation,
                                           GSList *key_values);

void paf_interpolation_set_key_value_options (PafInterpolation *interpolation,
                                              PafKeyValue *key_value,
                                              GValue *options);

void paf_interpolation_set_key_value_options_from_fraction (
                                              PafInterpolation *interpolation,
                                              gfloat fraction,
                                              GValue *options);

void paf_interpolation_add_key_value_from_value (
                                                PafInterpolation *interpolation,
                                                gfloat fraction,
                                                GValue *value);

void paf_interpolation_add_key_value_from_float (
                                                PafInterpolation *interpolation,
                                                gfloat fraction,
                                                gfloat value);

void paf_interpolation_add_key_value_from_int (PafInterpolation *interpolation,
                                               gfloat fraction,
                                               gint value);

void paf_interpolation_modify_key_value (PafInterpolation *interpolation,
                                         gfloat fraction,
                                         PafKeyValue *key_value);

void paf_interpolation_remove_key_value_from_fraction (
                                             PafInterpolation *interpolation,
                                             gfloat fraction);

void paf_interpolation_remove_key_value (PafInterpolation *interpolation,
                                         PafKeyValue *value);

void paf_interpolation_get_surrounding_key_values (
                                                PafInterpolation *interpolation,
                                                gfloat fraction,
                                                PafKeyValue **before,
                                                PafKeyValue **after);

G_END_DECLS

#endif /* __PAF_INTERPOLATION_H__ */

