/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Operators
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

/**
 * SECTION:pafoperators
 * @short_description: FIXME
 * @see_also: #PafInterpolation.
 *
 * <refsect2>
 * The #PafOperators object is a "per-type" singleton. There is at most one
 * instance per #GType.
 * </refsect2>
 * FIXME
 */

#include "pafoperators.h"


G_DEFINE_TYPE (PafOperators, paf_operators, G_TYPE_OBJECT);

#define PAF_OPERATORS_GET_PRIVATE(o)\
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), PAF_TYPE_OPERATORS, PafOperatorsPrivate))

struct _PafOperatorsPrivate {
    PafOperatorsAddFunc         add_function;
    PafOperatorsScalarMultFunc  scalar_mult_function;
    PafOperatorsNormFunc        norm_function;
    PafOperatorsNewFunc         new_zero_function;
    PafOperatorsNewFunc         new_one_function;
};

/* PafOperators properties */
enum {
    TARGET_TYPE=1,
    LAST_PROPERTY
};

/* GObject stuff */

static void
paf_operators_set_property (GObject *object,
                            guint property_id,
                            const GValue *value,
                            GParamSpec *param_spec)
{
}

static void
paf_operators_get_property (GObject *object,
                            guint property_id,
                            GValue *value,
                            GParamSpec *param_spec)
{
}

static void
paf_operators_dispose (PafOperators *self)
{
}

static void
paf_operators_finalize (PafOperators *self)
{
}

static void
paf_operators_init (PafOperators *self)
{
    PafOperatorsPrivate *priv;

    priv = PAF_OPERATORS_GET_PRIVATE (self);
}

static void
paf_operators_class_init (PafOperatorsClass *self_class)
{
    GParamSpec *param_spec = NULL;
    GObjectClass *object_class = G_OBJECT_CLASS (self_class);

    g_type_class_add_private (self_class, sizeof (PafOperatorsPrivate));
    object_class->set_property = paf_operators_set_property;
    object_class->get_property = paf_operators_get_property;

    param_spec = g_param_spec_gtype ("target-type", "Target type",
                                     "The type on which these operations will "
                                     "work.",
                                     G_TYPE_NONE,
                                     G_PARAM_READWRITE
                                     | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property (object_class, TARGET_TYPE, param_spec);

    object_class->dispose =
        (void (*) (GObject *object)) paf_operators_dispose;
    object_class->finalize =
        (void (*) (GObject *object)) paf_operators_finalize;
}

/* Public methods */

void
paf_operators_register_base_types (void)
{
}

gboolean
paf_operators_register (GType type,
                        PafOperatorsAddFunc add_function,
                        PafOperatorsScalarMultFunc scalar_mult_function,
                        PafOperatorsNormFunc norm_function,
                        PafOperatorsNewFunc new_zero_function,
                        PafOperatorsNewFunc new_one_function)
{
    return FALSE;
}

PafOperators *
paf_operators_new (GType type)
{
    return NULL;
}

void
paf_operators_add (PafOperators *operators,
                   GValue *result, const GValue *value)
{
}

void
paf_operators_scalar_mult (PafOperators *operators,
                           GValue *result, gfloat scalar)
{
}

gfloat
paf_operators_norm (PafOperators *operators, const GValue *value)
{
    return 0.0f;
}

GValue *
paf_operators_create_zero  (PafOperators *operators)
{
    return NULL;
}

GValue *
paf_operators_create_one   (PafOperators *operators)
{
    return NULL;
}

