/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Operators
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

#ifndef __PAF_OPERATORS_H__
#define __PAF_OPERATORS_H__

#include "pafcommon.h"

G_BEGIN_DECLS

typedef struct _PafOperatorsPrivate PafOperatorsPrivate;

typedef struct {
  GObject parent;

  PafOperatorsPrivate *priv;
} PafOperators;

typedef struct {
  GObjectClass parent;
} PafOperatorsClass;

typedef void      (*PafOperatorsAddFunc)        (GValue *result,
                                                 const GValue *value);
typedef void      (*PafOperatorsScalarMultFunc) (GValue *result,
                                                 gfloat scalar);
typedef gfloat    (*PafOperatorsNormFunc)       (const GValue *value);
typedef GValue *  (*PafOperatorsNewFunc)    (void);

#define PAF_TYPE_OPERATORS (paf_operators_get_type ())
#define PAF_OPERATORS(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAF_TYPE_OPERATORS, PafOperators))
#define PAF_OPERATORS_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_CAST ((klass), PAF_TYPE_OPERATORS, PafOperatorsClass))
#define PAF_IS_OPERATORS(obj) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_OPERATORS))
#define PAF_IS_OPERATORS_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_TYPE ((klass), PAF_TYPE_OPERATORS))
#define PAF_OPERATORS_GET_CLASS(obj) \
    (G_TYPE_INSTANCE_GET_CLASS ((obj), PAF_TYPE_OPERATORS, PafOperatorsClass))

GType paf_operators_get_type (void);

void            paf_operators_register_base_types (void);

gboolean        paf_operators_register    (GType type,
                             PafOperatorsAddFunc add_function,
                             PafOperatorsScalarMultFunc scalar_mult_function,
                             PafOperatorsNormFunc norm_function,
                             PafOperatorsNewFunc new_zero_function,
                             PafOperatorsNewFunc new_one_function);

PafOperators *  paf_operators_new         (GType type);

void            paf_operators_add         (PafOperators *operators,
                                           GValue *result, const GValue *value);

void            paf_operators_scalar_mult (PafOperators *operators,
                                           GValue *result, gfloat scalar);

gfloat          paf_operators_norm        (PafOperators *operators,
                                           const GValue *value);

GValue *        paf_operators_create_zero (PafOperators *operators);
GValue *        paf_operators_create_one  (PafOperators *operators);

G_END_DECLS

#endif /* __PAF_OPERATORS_H__ */
