/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Transformation
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */

#ifndef __PAF_TRANSFORMATION_H__
#define __PAF_TRANSFORMATION_H__

#include "pafcommon.h"
#include "pafmodifier.h"

G_BEGIN_DECLS

typedef struct _PafTransformationPrivate PafTransformationPrivate;

/**
 * PafConditionFunc:
 * @data:
 * @value:
 */
typedef gboolean (*PafConditionFunc) (gpointer data, gpointer value);


typedef struct {
    GObject parent;

    PafTransformationPrivate *priv;
} PafTransformation;

typedef struct {
    GObjectClass parent;

    gboolean (*add_modifier)    (PafTransformation *transformation,
                                 PafModifier *modifier);
    gboolean (*remove_modifier) (PafTransformation *transformation,
                                 PafModifier *modifier);
    gboolean (*has_modifier)    (PafTransformation *transformation,
                                 const PafModifier *modifier);
    gboolean (*update)          (PafTransformation *transformation,
                                 gfloat fraction);
    void     (*set_trigger)     (PafTransformation *transformation,
                                 PafConditionFunc condition,
                                 gpointer user_data,
                                 GCallback callback);
} PafTransformationClass;

#define PAF_TYPE_TRANSFORMATION (paf_transformation_get_type ())
#define PAF_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAF_TYPE_TRANSFORMATION, PafTransformation))
#define PAF_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PAF_TYPE_TRANSFORMATION, PafTransformationClass))
#define PAF_IS_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAF_TYPE_TRANSFORMATION))
#define PAF_IS_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PAF_TYPE_TRANSFORMATION))
#define PAF_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PAF_TYPE_TRANSFORMATION, PafTransformationClass))


GType paf_transformation_get_type (void);

gboolean paf_transformation_add_modifier    (PafTransformation *transformation,
                                             PafModifier *modifier);
gboolean paf_transformation_remove_modifier (PafTransformation *transformation,
                                             PafModifier *modifier);
gboolean paf_transformation_has_modifier   (PafTransformation *transformation,
                                            const PafModifier *modifier);

gboolean paf_transformation_update          (PafTransformation *transformation,
                                             gfloat fraction);

void     paf_transformation_set_trigger     (PafTransformation *transformation,
                                             PafConditionFunc *condition,
                                             gpointer user_data,
                                             GCallback callback);


G_END_DECLS
#endif /* __PAF_TRANSFORMATION_H__ */
