/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Unit tests - PafClock
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */
#include "paftest.h"

/* FIXME:
 *  - add more stuff with transformations when transformation implementations
 *  actually exist
 *  - add comments explaining what function is tested where.
 */

/* init / cleanup stuff, not used for construction tests */

static PafClock *gClock = NULL;

static int
suite_init (void)
{
    gClock = paf_clock_new (1000);

    if (NULL == gClock)
        return 1;

    return 0;
}

static int
suite_cleanup (void)
{
    g_object_unref (gClock);

    return 0;
}

/* construction tests */

PAF_TEST_CONSTRUCT_DESTRUCT (PafClock, CLOCK, paf_clock_new (1001))

static void
test_new (void)
{
    PafClock *clock = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock = paf_clock_new (1000));

    g_object_unref (clock);
    clock = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock = paf_clock_new (0));
    g_object_unref (clock);
    clock = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock = paf_clock_new (G_MAXUINT));
    g_object_unref (clock);
}

static void
test_new_full_fail (void)
{
    G_GNUC_UNUSED PafClock *clock = NULL; /* FIXME: when PAF_TEST_ASSERT_WARNING
                                             is implemented */
    PafClock *clock0 = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock0 = paf_clock_new (1000));

    PAF_TEST_ASSERT_WARNING (clock =
                             paf_clock_new_full (1000,
                                                 -1.0f, /* should fail */
                                                 TRUE,
                                                 NULL,
                                                 NULL));

    PAF_TEST_ASSERT_WARNING (clock =
                             paf_clock_new_full (1000,
                                                 1.0f,
                                                 TRUE,
                                                 (PafTransformation *) clock0,
                                                 /* should fail */
                                                 NULL));

    PAF_TEST_ASSERT_WARNING (clock =
                             paf_clock_new_full (1000,
                                                 1.0f,
                                                 TRUE,
                                                 NULL,
                                                 (PafTicker *) clock0
                                                 /* should fail */));

    g_object_unref (clock0);
}

static void
test_new_full (void)
{
    PafClock *clock = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      1.0f,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (0,
                                                      1.0f,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);
    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (G_MAXUINT,
                                                      1.0f,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      0.0f,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      G_MINFLOAT,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      G_MAXFLOAT,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      PAF_INFINITY,
                                                      TRUE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                  paf_clock_new_full (1000,
                                                      1.0f,
                                                      FALSE,
                                                      NULL,
                                                      NULL));
    g_object_unref (clock);

    {
        PafTicker *ticker = NULL;
        CU_ASSERT_PTR_NOT_NULL (ticker = paf_ticker_new ());
        CU_ASSERT_PTR_NOT_NULL_FATAL (clock =
                                      paf_clock_new_full (1000,
                                                          1.0f,
                                                          TRUE,
                                                          NULL,
                                                          ticker));
        g_object_unref (clock);
    }
}


/* property tests */

static void
test_auto_reverse (PafClock *clock)
{
    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, auto_reverse, TRUE);

    PAF_TEST_CHECK_PROPERTY (clock, clock, auto_reverse, TRUE);
    PAF_TEST_CHECK_PROPERTY (clock, clock, auto_reverse, FALSE);
}

static void
test_duration (PafClock *clock)
{
    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, duration, 1234);

    PAF_TEST_CHECK_PROPERTY (clock, clock, duration, 1234);
    PAF_TEST_CHECK_PROPERTY (clock, clock, duration, 1000);
    PAF_TEST_CHECK_PROPERTY (clock, clock, duration, 0);
    PAF_TEST_CHECK_PROPERTY (clock, clock, duration, G_MAXUINT);

}

static void
test_repeat_count (PafClock *clock)
{
    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, repeat_count, 1.0f);

    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, repeat_count, -1.0f);
    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, repeat_count, -G_MAXFLOAT);
    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, repeat_count, -G_MINFLOAT);

    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, 0.0f);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, G_MINFLOAT);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, 0.5f);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, 1.0f);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, 3.0f);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, 123.456f);
    PAF_TEST_CHECK_PROPERTY (clock, clock, repeat_count, G_MAXFLOAT);
}

static void
test_state (PafClock *clock)
{
    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, state, PAF_STATE_STOPPED);

    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, state, PAF_LAST_STATE);
    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, state, PAF_LAST_STATE +1);
    PAF_TEST_CHECK_PROPERTY_FAIL (clock, clock, state, G_MAXINT);

    PAF_TEST_CHECK_PROPERTY (clock, clock, state, PAF_STATE_RUNNING);
    PAF_TEST_CHECK_PROPERTY (clock, clock, state, PAF_STATE_PAUSED);
    PAF_TEST_CHECK_PROPERTY (clock, clock, state, PAF_STATE_STOPPED);
}

/* FIXME: check behaviour with implemented transformations */
static void
test_time_transformation (PafClock *clock)
{
    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, time_transformation,
                                  NULL);

    PAF_TEST_CHECK_PROPERTY (clock, clock, time_transformation, NULL);
}

static void
test_ticker_property (PafClock *clock)
{
    PafTicker *ticker = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL (ticker = paf_ticker_new ());

    PAF_TEST_CHECK_SETTER_WARNING (NULL, clock, ticker, NULL);

    PAF_TEST_CHECK_PROPERTY (clock, clock, ticker, ticker);

    PAF_TEST_CHECK_PROPERTY (clock, clock, ticker, NULL);

    PAF_TEST_ASSERT_NO_WARNING (g_object_unref (ticker));

    ticker = paf_get_default_ticker ();

    PAF_TEST_CHECK_PROPERTY (clock, clock, ticker, ticker);

}

static void
test_properties (void)
{
    test_auto_reverse (gClock);
    test_duration (gClock);
    test_repeat_count (gClock);
    test_state (gClock);
    test_time_transformation (gClock);
    test_ticker_property (gClock);
}

/* signal tests */

static gboolean
state_changed_callback (PafClock *clock, guint state, gpointer user_data)
{
    PafState *ret_state = user_data;

    *ret_state = state;

    return TRUE;
}

static void
test_state_changed (PafClock *clock)
{
    PafState state = PAF_LAST_STATE;
    gulong id;

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_STOPPED);

    id = g_signal_connect (clock, "state-changed",
                           G_CALLBACK (state_changed_callback), &state);
    CU_ASSERT_NOT_EQUAL (id, 0);

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_RUNNING);
    CU_ASSERT_EQUAL (state, PAF_STATE_RUNNING);

    g_signal_handler_disconnect (clock, id);

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_STOPPED);
}



static gboolean
ticked_callback (PafClock *clock, gfloat fraction, gpointer user_data)
{
    gfloat *ret_fraction = user_data;

    *ret_fraction = fraction;

    return TRUE;
}

static void
test_ticked (PafClock *clock)
{
    gfloat fraction = -1.0f;
    gulong id;

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_STOPPED);

    id = g_signal_connect (clock, "ticked",
                           G_CALLBACK (ticked_callback), &fraction);
    CU_ASSERT_NOT_EQUAL (id, 0);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (clock));
    CU_ASSERT_EQUAL (fraction, -1.0f);

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_RUNNING);
    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (clock));
    CU_ASSERT_NOT_EQUAL (fraction, -1.0f);

    fraction = -1.0f;

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_PAUSED);
    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (clock));
    CU_ASSERT_EQUAL (fraction, -1.0f);


    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_RUNNING);
    g_signal_handler_disconnect (clock, id);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (clock));
    CU_ASSERT_EQUAL (fraction, -1.0f);

    PAF_TEST_CHECK_SETTER (clock, clock, state, PAF_STATE_STOPPED);
}

static void
test_signals (void)
{
    test_state_changed (gClock);
    test_ticked (gClock);
}

static void
test_fraction_value_simple (void)
{
    gulong id = 0;
    gfloat fraction = -1.0f;
    static gfloat epsilon = 0.001f;
    static guint duration = 1000;

    PAF_TEST_CHECK_SETTER (gClock, clock, duration, duration);
    PAF_TEST_CHECK_SETTER (gClock, clock, state, PAF_STATE_STOPPED);
    PAF_TEST_CHECK_SETTER (gClock, clock, ticker, NULL);
    PAF_TEST_CHECK_SETTER (gClock, clock, auto_reverse, FALSE);
    PAF_TEST_CHECK_SETTER (gClock, clock, time_transformation, NULL);
    PAF_TEST_CHECK_SETTER (gClock, clock, repeat_count, 1.0f);
    PAF_TEST_CHECK_SETTER (gClock, clock, state, PAF_STATE_RUNNING);

    id = g_signal_connect (gClock, "ticked",
                           G_CALLBACK (ticked_callback), &fraction);
    CU_ASSERT_NOT_EQUAL (id, 0);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.0f);
    CU_ASSERT (fraction < epsilon);

    g_usleep (250 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.25f - epsilon);
    CU_ASSERT (fraction <= 0.25f + epsilon);

    g_usleep (10 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.26f - epsilon);
    CU_ASSERT (fraction <= 0.26f + epsilon);

    g_usleep (730 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.99f - epsilon);
    CU_ASSERT (fraction <= 0.99f + epsilon);

    g_usleep (9.9 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 1.0f - epsilon);
    CU_ASSERT (fraction <= 1.0f + epsilon);

    g_usleep (10 * duration);

    fraction = -1.0f;

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));

    CU_ASSERT_EQUAL (fraction, -1.0f);

    PAF_TEST_ASSERT_NO_WARNING (g_signal_handler_disconnect (gClock, id));
}

static void
do_fraction_value_repeat (gboolean auto_reverse)
{
    gulong id = 0;
    gfloat fraction = -1.0f;
    static gfloat epsilon = 0.001f;
    static guint duration = 500;

    PAF_TEST_CHECK_SETTER (gClock, clock, duration, duration);
    PAF_TEST_CHECK_SETTER (gClock, clock, state, PAF_STATE_STOPPED);
    PAF_TEST_CHECK_SETTER (gClock, clock, ticker, NULL);
    PAF_TEST_CHECK_SETTER (gClock, clock, auto_reverse, auto_reverse);
    PAF_TEST_CHECK_SETTER (gClock, clock, time_transformation, NULL);
    PAF_TEST_CHECK_SETTER (gClock, clock, repeat_count, 2.5f);

    id = g_signal_connect (gClock, "ticked",
                           G_CALLBACK (ticked_callback), &fraction);
    CU_ASSERT_NOT_EQUAL (id, 0);

    PAF_TEST_CHECK_SETTER (gClock, clock, state, PAF_STATE_RUNNING);
    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.0f);
    CU_ASSERT (fraction < epsilon);

    g_usleep (250 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.25f - epsilon);
    CU_ASSERT (fraction <= 0.25f + epsilon);

    g_usleep (1000 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    if (auto_reverse)
    {
        CU_ASSERT (fraction >= 0.75f - epsilon);
        CU_ASSERT (fraction <= 0.75f + epsilon);
    }
    else
    {
        CU_ASSERT (fraction >= 0.25f - epsilon);
        CU_ASSERT (fraction <= 0.25f + epsilon);
    }

    fraction = -1.0f;
    g_usleep (1000 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.25f - epsilon);
    CU_ASSERT (fraction <= 0.25f + epsilon);

    g_usleep (249.9 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT (fraction >= 0.5f - epsilon);
    CU_ASSERT (fraction <= 0.5f + epsilon);

    fraction = -1.0f;
    g_usleep (100 * duration);

    PAF_TEST_ASSERT_NO_WARNING (paf_clock_tick (gClock));
    CU_ASSERT_EQUAL (fraction, -1.0f);

    PAF_TEST_ASSERT_NO_WARNING (g_signal_handler_disconnect (gClock, id));
}

static void
test_fraction_value_repeat (void)
{
    do_fraction_value_repeat (FALSE);
    do_fraction_value_repeat (TRUE);
}

void
add_pafclock_suite (void)
{
    CU_pSuite suite = paf_test_add_suite ("PafClock-noinit", NULL, NULL);
    PAF_TEST_ADD_TEST (suite, construct_destruct);
    PAF_TEST_ADD_TEST (suite, test_new);
    PAF_TEST_ADD_TEST (suite, test_new_full_fail);
    PAF_TEST_ADD_TEST (suite, test_new_full);

    suite = paf_test_add_suite ("PafClock",
                                          suite_init, suite_cleanup);
    PAF_TEST_ADD_TEST (suite, test_properties);
    PAF_TEST_ADD_TEST (suite, test_signals);
    PAF_TEST_ADD_TEST (suite, test_fraction_value_simple);
    PAF_TEST_ADD_TEST (suite, test_fraction_value_repeat);
}

