/* vi:expandtab:ts=4:sw=4:cindent:tw=80:cinoptions=(0,u0,t0
 *
 * PAF Animation Framework - Unit tests
 *
 * Copyright © 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Guillaume Emont <guillaume@fluendo.com>
 */
#ifndef __PAF_TEST_H__
#define __PAF_TEST_H__

#include <paf/paf.h>
#include <CUnit/CUnit.h>

#define PAF_TEST_ADD_TEST(suite, test) \
    (paf_test_add_test (suite, #test, test))

#define PAF_TEST_CLEAR_WARNING_FLAG /* FIXME */
#define PAF_TEST_ASSERT_NO_WARNING_HAPPENED /* FIXME */

/**
 * PAF_TEST_ASSERT_WARNING:
 * @expression: a valid C expression
 *
 * Executes @expression and checks that no g_warning happened during that
 * execution.
 */
/* FIXME */
#define PAF_TEST_ASSERT_WARNING(expression) do { \
    (void) 0; \
}while (0)

/**
 * PAF_TEST_ASSERT_NO_WARNING:
 * @expression: a valid C expression
 *
 * Executes @expression and checks that a g_warning happened during that
 * execution.
 */
/* FIXME */
#define PAF_TEST_ASSERT_NO_WARNING(expression) do { \
    expression; \
}while (0)

/**
 * PAF_TEST_CONSTRUCT_DESTRUCT:
 * @Type: The C type of the GObject to be tested.
 * @TYPE: The capitalised name of the type without the PAF_ prefix (e.g.  for
 * PafClock -> CLOCK.
 * @construction_expression: the expression that creates and returns the object.
 *
 * Expands to the implementation of a static test method called
 * construct_destruct() that tests the construction and destruction of an
 * object.
 */
/* FIXME: remove the Type parameter */
#define PAF_TEST_CONSTRUCT_DESTRUCT(Type, TYPE, construction_expression) \
    static void \
    construct_destruct (void) \
    { \
        Type *object; \
 \
        CU_ASSERT_PTR_NOT_NULL_FATAL (object = (construction_expression)); \
        CU_ASSERT (G_IS_OBJECT (object)); \
        CU_ASSERT (PAF_IS_##TYPE (object)); \
 \
        PAF_TEST_ASSERT_NO_WARNING (g_object_unref (object)); \
    }


/**
 * PAF_TEST_CHECK_SETTER:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to set @property with @value on @object and check there is no warning.
 */
#define PAF_TEST_CHECK_SETTER(object, object_name, property, value) \
    PAF_TEST_ASSERT_NO_WARNING ( \
        paf_##object_name##_set_##property (object, value))

/**
 * PAF_TEST_CHECK_SETTER_WARNING:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to set @property with @value on @object and check that a warning
 * happenned.
 */
#define PAF_TEST_CHECK_SETTER_WARNING(object, object_name, property, value) \
    PAF_TEST_ASSERT_WARNING ( \
        paf_##object_name##_set_##property (object, value))

/**
 * PAF_TEST_CHECK_GETTER:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to get @property from @object and check it is equal to @value (with a
 * `==` comparison).
 */
#define PAF_TEST_CHECK_GETTER(object, object_name, property, value) \
    CU_ASSERT (paf_##object_name##_get_##property (object) == value)

/**
 * PAF_TEST_CHECK_GETTER_FAIL:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to get @property from @object and check it is not equal to @value (with
 * a `!=` comparison).
 */
#define PAF_TEST_CHECK_GETTER_FAIL(object, object_name, property, value) \
    CU_ASSERT (paf_##object_name##_get_##property (object) != value)

/**
 * PAF_TEST_CHECK_PROPERTY:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to set @value to the property @property of @object, then try to get
 * @property and checks it has the expected value.
 */
#define PAF_TEST_CHECK_PROPERTY(object, object_name, property, value) \
    do { \
        PAF_TEST_CHECK_SETTER (object, object_name, property, value); \
        PAF_TEST_CHECK_GETTER (object, object_name, property, value); \
    } while (0)

/**
 * PAF_TEST_CHECK_PROPERTY_FAIL:
 * @object:
 * @object_name:
 * @property:
 * @value:
 *
 * Try to set @value to the property @property of @object and checks that it
 * failed with a warning.
 */
#define PAF_TEST_CHECK_PROPERTY_FAIL(object, object_name, property, value) \
    do { \
        PAF_TEST_CHECK_SETTER_WARNING (object, object_name, property, value); \
        PAF_TEST_CHECK_GETTER_FAIL (object, object_name, property, value); \
    } while (0)


void add_pafticker_suite    (void);
void add_pafclock_suite     (void);

CU_pSuite paf_test_add_suite    (const char *suite_name,
                                 CU_InitializeFunc init,
                                 CU_CleanupFunc cleanup);

void paf_test_add_test          (CU_pSuite suite,
                                 const char *test_name,
                                 CU_TestFunc test);

int paf_test_init_check_warnings (void);
int paf_test_cleanup_check_warnings (void);

#endif /* __PAF_TEST_H__ */
